﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace RedRat.Samples.RR_CLI
{
    class ArgumentParser
    {
        private readonly Dictionary<string, Argument> arguments = new Dictionary<string, Argument>( StringComparer.InvariantCultureIgnoreCase );

        public static string BlinkArg = "-blink";
        public static string CaptureArg = "-capture";
        public static string OutputArg = "-output";
        public static string PrintArg = "-print";
        public static string DeviceArg = "-device";
        public static string SignalArg = "-signal";
        public static string XmlArg = "-xml";

        public ArgumentParser()
        {
            AddArgument( new Argument( BlinkArg, null ) );
            AddArgument( new Argument( CaptureArg, "filename" ) );
            AddArgument( new Argument( OutputArg, "filename" ) );
            AddArgument( new Argument( PrintArg, null ) );
            AddArgument( new Argument( DeviceArg, "devicename" ) );
            AddArgument( new Argument( SignalArg, "signalname" ) );
            AddArgument( new Argument( XmlArg, null ) );
        }

        void AddArgument( Argument arg )
        {
            arguments.Add( arg.Name, arg );
        }

        public bool ParseArgs( string[] args, out string message )
        {
            var argCount = args.Length;
            if ( argCount < 2 )
            {
                message = $"Require at least 2 arguments, got {argCount}";
                return false;
            }

            RedRatName = args[ 0 ];

            for ( var pos = 1; pos < argCount; ++pos )
            {
                var argName = args[ pos ];
                if ( argName.StartsWith( "-" ) )
                {
                    if ( arguments.TryGetValue( argName, out var arg ) )
                    {
                        if ( arg.RequiresOption )
                        {
                            if ( pos < argCount - 1 )
                            {
                                ++pos;
                                arg.OptionValue = args[ pos ];
                            }
                            else
                            {
                                message = $"Argument {argName} requires {arg.OptionName} option";
                                return false;
                            }
                        }
                        arg.Present = true;
                    }
                    else
                    {
                        message = "Unknown argument " + argName;
                        return false;
                    }
                }
                else
                {
                    message = "Unexpected option " + argName;
                    return false;
                }
            }

            message = string.Empty;
            return true;
        }

        public string RedRatName { get; private set; }

        public bool HaveArg( string argName )
        {
            return arguments[ argName ].Present;
        }

        public bool HavePrintArg => HaveArg( PrintArg );

        public bool ShouldBlink => HaveArg( BlinkArg );

        public bool ShouldCapture => HaveArg( CaptureArg );

        public bool ShouldOutput => HaveArg( OutputArg );

        public string GetArgOption( string argName )
        {
            return arguments[ argName ].OptionValue;
        }
    }
}
