﻿using System;
using System.Linq;
using RedRat.IR;
using RedRat.Util;
using RedRat.AVDeviceMngmt;

namespace RedRat.Samples.RR_CLI
{
    class XmlSignalLoader
    {
        public static IRPacket LoadSignal( string fileName, string deviceName, string signalName, out string message )
        {
            /*
             * xml file could be one of four things:
             * 1. IRPacket (most likely as previous versions only understood this format)
             * 2. AVDeviceDB (needs device and signal options)
             * 3. AVDevice (needs signal option)
             * 4. Some other xml format
             */
            var result = GenericXmlSerializer<IRPacket>.FromFile( fileName );
            if ( result.Successful )
            {
                message = string.Empty;
                return result.Object;
            }
            else
            {
                var haveDeviceName = !string.IsNullOrEmpty( deviceName );
                var haveSignalName = !string.IsNullOrEmpty( signalName );

                if ( result.EncounteredType == "AVDeviceDB" )
                {
                    if ( haveDeviceName && haveSignalName )
                    {
                        var db = GenericXmlSerializer<AVDeviceDB>.FromFile( fileName ).Object;
                        var device = db.AVDevices.FirstOrDefault( d => string.Equals( d.Name, deviceName, StringComparison.InvariantCultureIgnoreCase ) );
                        if ( device != null )
                        {
                            var signal = device.GetSignal( signalName );
                            message = signal == null ? $"Couldn't find signal {signalName}" : string.Empty;
                            return signal;
                        }
                        else
                        {
                            message = "Couldn't find device " + deviceName;
                        }
                    }
                    else
                    {
                        message = "Supplied xml file contains a set of devices and signals. You must specify both the -device and -signal options to identify the signal to use";
                    }
                }
                else if ( result.EncounteredType == "AVDevice" )
                {
                    if ( haveSignalName )
                    {
                        var device = GenericXmlSerializer<AVDevice>.FromFile( fileName ).Object;
                        var signal = device.GetSignal( signalName );
                        message = signal == null ? "Couldn't find signal " + signalName : string.Empty;
                        return signal;
                    }
                    else
                    {
                        message = "Supplied xml file contains a set of signals. You must specify the -signal option to identify the signal to use";
                    }
                }
                else
                {
                    message = $"Failed to load {fileName}. xml files must contain AVDeviceDB, AVDevice or IRPacket. Found {result.EncounteredType}";
                }
            }

            return null;
        }
    }
}
