using System;
using System.Windows.Forms;
using RedRat.IR;
using RedRat.RedRat3;

namespace RedRat.TestRemote {

	/// <summary>
	/// Allows configuration of properties associated with a button.
	/// </summary>
    public class ButtonPropertiesDialog : Form
    {
        private Label label1;
        private TextBox buttonTextBox;
        private Button learnIRButton;
        private Button okButton;
        private Button cancelButton;
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private readonly System.ComponentModel.Container components = null;

        protected IRButton thisButton;

        protected global::RedRat.TestRemote.TestRemote parent;

        protected IRPacket irPacket;

        protected bool changed;

        public ButtonPropertiesDialog(IRButton initIRButton, global::RedRat.TestRemote.TestRemote initParent)
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            thisButton = initIRButton;
            parent = initParent;

            // Set GUI state from button
            buttonTextBox.Text = thisButton.Text;
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                components?.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.label1 = new System.Windows.Forms.Label();
            this.buttonTextBox = new System.Windows.Forms.TextBox();
            this.learnIRButton = new System.Windows.Forms.Button();
            this.okButton = new System.Windows.Forms.Button();
            this.cancelButton = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(16, 24);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(72, 16);
            this.label1.TabIndex = 0;
            this.label1.Text = "Button Text:";
            this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // buttonTextBox
            // 
            this.buttonTextBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.buttonTextBox.Location = new System.Drawing.Point(104, 24);
            this.buttonTextBox.Name = "buttonTextBox";
            this.buttonTextBox.TabIndex = 1;
            this.buttonTextBox.Text = "";
            this.buttonTextBox.TextChanged += new System.EventHandler(this.ButtonTextBoxTextChanged);
            // 
            // learnIRButton
            // 
            this.learnIRButton.Location = new System.Drawing.Point(72, 64);
            this.learnIRButton.Name = "learnIRButton";
            this.learnIRButton.TabIndex = 2;
            this.learnIRButton.Text = "Learn IR";
            this.learnIRButton.Click += new System.EventHandler(this.LearnIRButtonClick);
            // 
            // okButton
            // 
            this.okButton.Location = new System.Drawing.Point(24, 112);
            this.okButton.Name = "okButton";
            this.okButton.TabIndex = 3;
            this.okButton.Text = "OK";
            this.okButton.Click += new System.EventHandler(this.OkButtonClick);
            // 
            // cancelButton
            // 
            this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.cancelButton.Location = new System.Drawing.Point(136, 112);
            this.cancelButton.Name = "cancelButton";
            this.cancelButton.TabIndex = 4;
            this.cancelButton.Text = "Cancel";
            // 
            // ButtonPropertiesDialog
            // 
            this.AcceptButton = this.okButton;
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.CancelButton = this.cancelButton;
            this.ClientSize = new System.Drawing.Size(232, 150);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                  this.cancelButton,
                                                                  this.okButton,
                                                                  this.learnIRButton,
                                                                  this.buttonTextBox,
                                                                  this.label1});
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.Name = "ButtonPropertiesDialog";
            this.Text = "Properties";
            this.ResumeLayout(false);

        }
        #endregion

        private void OkButtonClick(object sender, EventArgs e)
        {
            // Set IRButton state from info here..
            thisButton.Text = buttonTextBox.Text;
            thisButton.Info.IRPacket = irPacket;
            parent.Changed = changed;
            Close();
        }

        private void ButtonTextBoxTextChanged(object sender, EventArgs e)
        {
            changed = true;
        }

        /// <summary>
        /// Capture/learn a remote control signal.
        /// </summary>
        private void LearnIRButtonClick(object sender, EventArgs e)
        {
            var rr3 = parent.RedRat3;
            if (rr3 == null)
            {
                return;
            }

            SignalInputDialog signalInputDialog = null;

            // == Right, this is probably the most complicated bit of interaction ==
            // == with the RR3 in this program - but its not too bad really.      ==
            try
            {
                // 1. Tell the RR3 we want to input a demodulated signal (learn)
                rr3.GetModulatedSignal(0);

                // 2. Create modal dialog that allows us to cancel the operation if required.
                signalInputDialog = new SignalInputDialog(rr3);

                // 3. The dialog box has to handle the input signal event from the RR3.
                rr3.LearningSignalIn += signalInputDialog.SignalDataIn;

                // 4. Popup dialog....
                signalInputDialog.ShowDialog(this);

                // 5. Once finished, get the event object
                var siea = signalInputDialog.SignalInEvent;

                switch (siea.Action)
                {
                    // Have input data of some kind...
                    case SignalEventAction.MODULATED_SIGNAL:
                        irPacket = siea.IRPacket;
                        changed = true;
                        break;

                    // We have had some error from the comms with the USB device, so read exception and throw it.
                    case SignalEventAction.EXCEPTION:
                        throw siea.Exception;

                    // The user has pressed cancel
                    case SignalEventAction.INPUT_CANCELLED:
                        break;
                }
            }
            catch (Exception ex)
            {
                var msg = ex.Message;
                if (ex.InnerException != null)
                {
                    msg += ": " + ex.InnerException.Message;
                }
                MessageBox.Show(msg, "Communication Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            finally
            {
                // Remove listener...
                if (signalInputDialog != null)
                {
                    rr3.LearningSignalIn -= signalInputDialog.SignalDataIn;
                }

                signalInputDialog?.Close();
            }
        }
    }
}