using System;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using RedRat.Config;
using RedRat.IR;
using RedRat.RedRat3;
using RedRat.RedRat3.USB;
using RedRat.USB;

namespace RedRat.TestRemote
{

    /// <summary>
    /// This program shows how to use the RedRat3 to build a simple virtual remote control. There are a number 
    /// of buttons, each of which can have a single IR remote control signal associated with it.
    /// 
    /// By right clicking on a button, the button's text can be changed and a new IR signal learned.
    /// </summary>
    public class TestRemote : Form
    {
        // The actual RedRat3 we're using...
        protected IRedRat3 redRat3;

        // Registry info. for config file
        protected const string testRemoteKey = "TestRemote";
        protected const string configFileKeyName = "ConfigFile";
        protected ConfigValue configFileRK;

        private IRButton button1;
        private IRButton button2;
        private IRButton button3;
        private IRButton button4;
        private IRButton button5;
        private IRButton button6;
        private IRButton button7;
        private IRButton button8;
        private IRButton button9;
        private IRButton button10;
        private MainMenu mainMenu;
        private MenuItem fileMenuItem;
        private MenuItem helpMenuItem;
        private MenuItem aboutMenuItem;
        private MenuItem openMenuItem;
        private MenuItem saveMenuItem;
        private MenuItem exitMenuItem;
        private MenuItem menuItem1;
        private MenuItem rr3InfoMenuItem;

        /// <summary>
        /// Required designer variable.
        /// </summary>
        private readonly System.ComponentModel.Container components = null;

        public TestRemote()
        {
            try
            {
                //
                // Required for Windows Form Designer support
                //
                InitializeComponent();

                // Find RedRat3s.
                OpenRedRat3();

                // Load default state
                configFileRK = Configuration.Provider.GetOrCreateValue(testRemoteKey, configFileKeyName, "", false);

                if (((string)configFileRK.Value).Equals("")) return;
                try
                {
                    OpenFile((string)configFileRK.Value);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                // Catch-all handler.
                var msg = $"Error occurred in application: {ex}";
                MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {

                // Clear up RR3s and their resources.
                RedRat3USBImpl.DisposeOfAll();

                components?.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.button1 = new RedRat.TestRemote.IRButton();
            this.button2 = new RedRat.TestRemote.IRButton();
            this.button3 = new RedRat.TestRemote.IRButton();
            this.button4 = new RedRat.TestRemote.IRButton();
            this.button5 = new RedRat.TestRemote.IRButton();
            this.button6 = new RedRat.TestRemote.IRButton();
            this.button7 = new RedRat.TestRemote.IRButton();
            this.button8 = new RedRat.TestRemote.IRButton();
            this.button9 = new RedRat.TestRemote.IRButton();
            this.button10 = new RedRat.TestRemote.IRButton();
            this.mainMenu = new System.Windows.Forms.MainMenu();
            this.fileMenuItem = new System.Windows.Forms.MenuItem();
            this.openMenuItem = new System.Windows.Forms.MenuItem();
            this.saveMenuItem = new System.Windows.Forms.MenuItem();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.exitMenuItem = new System.Windows.Forms.MenuItem();
            this.helpMenuItem = new System.Windows.Forms.MenuItem();
            this.aboutMenuItem = new System.Windows.Forms.MenuItem();
            this.rr3InfoMenuItem = new System.Windows.Forms.MenuItem();
            this.SuspendLayout();
            // 
            // button1
            // 
            this.button1.Location = new System.Drawing.Point(8, 16);
            this.button1.Name = "button1";
            this.button1.Size = new System.Drawing.Size(56, 23);
            this.button1.TabIndex = 0;
            this.button1.Click += new System.EventHandler(this.ButtonClick);
            this.button1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button2
            // 
            this.button2.Location = new System.Drawing.Point(80, 16);
            this.button2.Name = "button2";
            this.button2.Size = new System.Drawing.Size(56, 23);
            this.button2.TabIndex = 1;
            this.button2.Click += new System.EventHandler(this.ButtonClick);
            this.button2.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button3
            // 
            this.button3.Location = new System.Drawing.Point(8, 56);
            this.button3.Name = "button3";
            this.button3.Size = new System.Drawing.Size(56, 23);
            this.button3.TabIndex = 2;
            this.button3.Click += new System.EventHandler(this.ButtonClick);
            this.button3.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button4
            // 
            this.button4.Location = new System.Drawing.Point(80, 56);
            this.button4.Name = "button4";
            this.button4.Size = new System.Drawing.Size(56, 23);
            this.button4.TabIndex = 3;
            this.button4.Click += new System.EventHandler(this.ButtonClick);
            this.button4.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button5
            // 
            this.button5.Location = new System.Drawing.Point(8, 96);
            this.button5.Name = "button5";
            this.button5.Size = new System.Drawing.Size(56, 23);
            this.button5.TabIndex = 4;
            this.button5.Click += new System.EventHandler(this.ButtonClick);
            this.button5.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button6
            // 
            this.button6.Location = new System.Drawing.Point(80, 96);
            this.button6.Name = "button6";
            this.button6.Size = new System.Drawing.Size(56, 23);
            this.button6.TabIndex = 5;
            this.button6.Click += new System.EventHandler(this.ButtonClick);
            this.button6.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button7
            // 
            this.button7.Location = new System.Drawing.Point(8, 136);
            this.button7.Name = "button7";
            this.button7.Size = new System.Drawing.Size(56, 23);
            this.button7.TabIndex = 6;
            this.button7.Click += new System.EventHandler(this.ButtonClick);
            this.button7.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button8
            // 
            this.button8.Location = new System.Drawing.Point(80, 136);
            this.button8.Name = "button8";
            this.button8.Size = new System.Drawing.Size(56, 23);
            this.button8.TabIndex = 7;
            this.button8.Click += new System.EventHandler(this.ButtonClick);
            this.button8.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button9
            // 
            this.button9.Location = new System.Drawing.Point(8, 176);
            this.button9.Name = "button9";
            this.button9.Size = new System.Drawing.Size(56, 23);
            this.button9.TabIndex = 8;
            this.button9.Click += new System.EventHandler(this.ButtonClick);
            this.button9.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // button10
            // 
            this.button10.Location = new System.Drawing.Point(80, 176);
            this.button10.Name = "button10";
            this.button10.Size = new System.Drawing.Size(56, 23);
            this.button10.TabIndex = 9;
            this.button10.Click += new System.EventHandler(this.ButtonClick);
            this.button10.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ButtonMouseDown);
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                             this.fileMenuItem,
                                                                             this.helpMenuItem});
            // 
            // fileMenuItem
            // 
            this.fileMenuItem.Index = 0;
            this.fileMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                 this.openMenuItem,
                                                                                 this.saveMenuItem,
                                                                                 this.menuItem1,
                                                                                 this.exitMenuItem});
            this.fileMenuItem.Text = "File";
            // 
            // openMenuItem
            // 
            this.openMenuItem.Index = 0;
            this.openMenuItem.Text = "Open";
            this.openMenuItem.Click += new System.EventHandler(this.OpenMenuItemClick);
            // 
            // saveMenuItem
            // 
            this.saveMenuItem.Index = 1;
            this.saveMenuItem.Text = "Save";
            this.saveMenuItem.Click += new System.EventHandler(this.SaveMenuItemClick);
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 2;
            this.menuItem1.Text = "-";
            // 
            // exitMenuItem
            // 
            this.exitMenuItem.Index = 3;
            this.exitMenuItem.Text = "Exit";
            this.exitMenuItem.Click += new System.EventHandler(this.ExitMenuItemClick);
            // 
            // helpMenuItem
            // 
            this.helpMenuItem.Index = 1;
            this.helpMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                 this.aboutMenuItem,
                                                                                 this.rr3InfoMenuItem});
            this.helpMenuItem.Text = "Help";
            // 
            // aboutMenuItem
            // 
            this.aboutMenuItem.Index = 0;
            this.aboutMenuItem.Text = "About";
            this.aboutMenuItem.Click += new System.EventHandler(this.AboutMenuItemClick);
            // 
            // rr3InfoMenuItem
            // 
            this.rr3InfoMenuItem.Index = 1;
            this.rr3InfoMenuItem.Text = "RedRat3 Info";
            this.rr3InfoMenuItem.Click += new System.EventHandler(this.Rr3InfoMenuItemClick);
            // 
            // TestRemote
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(144, 224);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                  this.button10,
                                                                  this.button9,
                                                                  this.button8,
                                                                  this.button7,
                                                                  this.button6,
                                                                  this.button5,
                                                                  this.button4,
                                                                  this.button3,
                                                                  this.button2,
                                                                  this.button1});
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
            this.Menu = this.mainMenu;
            this.Name = "TestRemote";
            this.Text = " TestRemote";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.TestRemoteClosing);
            this.ResumeLayout(false);

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.Run(new TestRemote());
        }

        /// <summary>
        /// Opens the first RedRat3 found.
        /// </summary>
        protected void OpenRedRat3()
        {
            try
            {
                // Find the no. of RR3s connected.
                var devices = RedRat3USBImpl.FindDevices();
                if (devices.Count > 0)
                {
                    // Just take the first device found.
                    redRat3 = (IRedRat3)devices[0].GetRedRat();
                }
                else
                {
                    MessageBox.Show("No RedRat3 devices found.", "TestRemote Warning",
                      MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        /// <summary>
        /// Gets/sets whether the state has changed.
        /// </summary>
        public bool Changed { get; set; }

        public IRedRat3 RedRat3 => redRat3;

        private void ButtonClick(object sender, EventArgs e)
        {
            // Output the signal...
            if (!(sender is IRButton)) return;

            var irPacket = ((IRButton)sender).Info.IRPacket;
            var sig = irPacket as ModulatedSignal;
            if (sig == null) return;
            if (redRat3 == null) return;
            try
            {
                // Send the ModulatedSignal object to the RR3
                redRat3.OutputModulatedSignal(sig);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void ButtonMouseDown(object sender, MouseEventArgs e)
        {
            // If right click, popup button properties dialog.
            if (e.Button != MouseButtons.Right) return;
            if (!( sender is IRButton button )) return;
            var dlg = new ButtonPropertiesDialog(button, this);
            dlg.ShowDialog();
        }

        private void ExitMenuItemClick(object sender, EventArgs e)
        {
            Close();
        }

        private void SaveMenuItemClick(object sender, EventArgs e)
        {
            SaveFile();
        }

        private void OpenMenuItemClick(object sender, EventArgs e)
        {
            // Read state from XML file.
            var openFileDialog = new OpenFileDialog
            {
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                RestoreDirectory = true
            };

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                OpenFile(openFileDialog.FileName);
            }
        }

        /// <summary>
        /// Loads program state from file.
        /// </summary>
        protected void OpenFile(string fileName)
        {
            var serializer = new XmlSerializer(typeof(ButtonInfo[]));
            var fileInfo = new FileInfo(fileName);
            FileStream fs = null;
            try
            {
                fs = new FileStream(fileInfo.FullName, FileMode.Open);
                var infoArr = (ButtonInfo[])serializer.Deserialize(fs);
                button1.Info = infoArr[0];
                button2.Info = infoArr[1];
                button3.Info = infoArr[2];
                button4.Info = infoArr[3];
                button5.Info = infoArr[4];
                button6.Info = infoArr[5];
                button7.Info = infoArr[6];
                button8.Info = infoArr[7];
                button9.Info = infoArr[8];
                button10.Info = infoArr[9];

                // Store file name in registry
                configFileRK.Value = fileInfo.FullName;

                // Reset flag
                Changed = false;

            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error opening config file: {ex.Message}", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            finally
            {
                fs?.Close();
            }
        }

        protected void SaveFile()
        {
            // Save to XML file
            var saveFileDialog = new SaveFileDialog
            {
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                RestoreDirectory = true
            };

            if (saveFileDialog.ShowDialog() != DialogResult.OK) return;

            // Put info into an array
            var infoArr = new ButtonInfo[10];
            infoArr[0] = button1.Info;
            infoArr[1] = button2.Info;
            infoArr[2] = button3.Info;
            infoArr[3] = button4.Info;
            infoArr[4] = button5.Info;
            infoArr[5] = button6.Info;
            infoArr[6] = button7.Info;
            infoArr[7] = button8.Info;
            infoArr[8] = button9.Info;
            infoArr[9] = button10.Info;

            // == Save the data in an XML file ==
            var fileInfo = new FileInfo(saveFileDialog.FileName);
            var ser = new XmlSerializer(typeof(ButtonInfo[]));
            var writer = new StreamWriter(fileInfo.FullName);
            ser.Serialize(writer, infoArr);
            writer.Close();

            // Store file name in registry
            configFileRK.Value = fileInfo.FullName;
        }

        private void TestRemoteClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (!Changed) return;

            var result = MessageBox.Show(this, "Configuration changed - do you want to save?",
                "Configuration Changed",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question, MessageBoxDefaultButton.Button1,
                MessageBoxOptions.RightAlign);

            if (result == DialogResult.Yes)
            {
                SaveFile();
            }
        }

        /// <summary>
        /// Put up "About" box.
        /// </summary>
        private void AboutMenuItemClick(object sender, EventArgs e)
        {
            // All info. obtained from the AssemblyInfo.cs file.
            var versionInfo = new StringBuilder();
            // Get version info.
            var assembly = Assembly.GetExecutingAssembly();

            // Title
            var ata = (AssemblyTitleAttribute[])assembly.GetCustomAttributes(typeof(AssemblyTitleAttribute), true);
            if (ata.Length > 0)
            {
                // Take the first one. Do we ever have more???
                versionInfo.Append(ata[0].Title + " - ");
            }

            // Description
            var ada = (AssemblyDescriptionAttribute[])assembly.GetCustomAttributes(typeof(AssemblyDescriptionAttribute), true);
            if (ada.Length > 0)
            {
                // Take the first one. Do we ever have more???
                versionInfo.Append(ada[0].Description + "\n");
            }

            // Version
            var version = assembly.GetName().Version;
            versionInfo.Append("Version: " + version.Major + "." + version.Minor);

            MessageBox.Show(versionInfo.ToString(), "About TestRemote", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Put up a dialog box to show information about RedRat3.
        /// </summary>
        private void Rr3InfoMenuItemClick(object sender, EventArgs e)
        {
            // Obtain and display info about the RR3.
            string msg;
            if (redRat3 == null)
            {
                msg = "No RedRat3 connected.";
            }
            else
            {
                // Obtain information from RR3 to display....
                var devInfo = (USBDeviceInfo)redRat3.DeviceInformation;
                var sb = new StringBuilder();
                sb.Append(devInfo + "\n");
                sb.Append("Hardware Version: " + devInfo.ProductName + "." + devInfo.ProductDescriptor.Version + "\n");
                sb.Append("Firmware Version: " + redRat3.FirmwareVersion);
                sb.Append("\nSerial Number: " + devInfo.SerialNumber);
                sb.Append("\nLocation: " + redRat3.LocationInformation);
                msg = sb.ToString();
            }
            MessageBox.Show(msg, "RedRat3 Info.", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}
