﻿Imports RedRat.Util
Imports RedRat.RedRat3
Imports System.Threading
Imports RedRat.AVDeviceMngmt
Imports System.IO
Imports System.Xml.Serialization

Namespace RedRat.Samples

    Module Module1

        Private myRedRat As IRedRat3
        Private deviceDB As AVDeviceDB
        Private dbFilename As String = "DeviceDB.xml"

        Sub Main()
            Try
                Console.WriteLine("RedRat VB.NET Sample")
                Console.WriteLine("--------------------")
                Console.WriteLine("")
                OpenRR3()

                If (myRedRat Is Nothing) Then
                    End
                End If

                ' Blink the red LED a few times. (Difficult to see as very dim through case)
                For value As Integer = 0 To 5
                    myRedRat.Blink()
                    Console.WriteLine("Blink!")
                    Thread.Sleep(500)
                Next

                ' Input some IR signals - wait 10s for this.
                ' Use lambda function
                AddHandler myRedRat.RCDetectorSignalIn, Sub(s, e)
                                                            Dim siea = CType(e, SignalEventArgs)
                                                            Dim action = siea.Action

                                                            If (action = SignalEventAction.RC_DETECTOR_ENABLED) Then
                                                                Console.WriteLine("RC detector enabled...")
                                                            ElseIf (action = SignalEventAction.RC_DETECTOR_DISABLED) Then
                                                                Console.WriteLine("RC detector enabled...")
                                                            ElseIf (action = SignalEventAction.MODULATED_SIGNAL) Then
                                                                Console.WriteLine("Have signal...")
                                                                ' Signal is in siea.ModulatedSignal property
                                                            End If
                                                        End Sub

                Console.WriteLine("")
                Console.WriteLine("Waiting 10s for the input of an IR signal...")
                myRedRat.RCDetectorEnabled = True
                Thread.Sleep(10000)
                myRedRat.RCDetectorEnabled = False


                ' Output first IR signal
                Dim deviceName = "CD"
                Dim signalName = "Play"
                Dim signal = GetSignal(deviceName, signalName)
                myRedRat.OutputModulatedSignal(signal)
                Console.WriteLine(String.Format("Output IR signal {0}->{1}.", deviceName, signalName))

                ' Wait a wee bit (1 second)
                Thread.Sleep(1000)

                ' Output a second IR signal
                signalName = "Pause"
                signal = GetSignal(deviceName, signalName)
                myRedRat.OutputModulatedSignal(signal)
                Console.WriteLine(String.Format("Output IR signal {0}->{1}.", deviceName, signalName))

            Catch Ex As Exception
                Console.WriteLine("Error in application: " & Ex.Message)
            End Try

        End Sub

        '
        ' Opens the first RedRat3 device found.
        '
        Private Sub OpenRR3()

            ' Find all RedRat3s attached to this computer.
            Dim locInfos = RRUtil.FindRedRats(LocationInfo.RedRatType.RedRat3)

            If (locInfos.Length = 0) Then
                Console.WriteLine("No RedRat3 devices found.")
                Return
            End If

            Console.WriteLine("No RedRat3 devices found.")

            ' We'll just take the first one.
            Console.WriteLine("Have found RedRat3: '" & locInfos(0).Name & "'")
            myRedRat = locInfos(0).GetRedRat
        End Sub

        '
        ' Provides an IR signal for output. First time through the signal DB file will be loaded.
        '
        Private Function GetSignal(deviceName As String, signalName As String)

            ' Load device DB containing IR signals if have not already done so.
            If (deviceDB Is Nothing) Then
                Dim fs As FileStream
                Try
                    Dim ser = New XmlSerializer(GetType([AVDeviceDB]))
                    fs = New FileStream((New FileInfo(dbFilename)).FullName, FileMode.Open)
                    deviceDB = ser.Deserialize(fs)
                Catch ex As Exception
                    Throw New Exception("Problem opening signal database file: " & ex.Message)
                    End
                Finally
                    If (fs IsNot Nothing) Then
                        fs.Close()
                    End If
                End Try
            End If

            ' Find the requested device and then the IR signal from this device
            Dim device = deviceDB.GetAVDevice(deviceName)
            If (device Is Nothing) Then
                Throw New Exception("Unable to find device in signal data file: " & deviceName)
            End If

            Dim signal = device.GetSignal(signalName)
            If (signal Is Nothing) Then
                Throw New Exception(String.Format("Unable to find signal {0} in device {1}.", signalName, deviceName))
            End If

            Return signal
        End Function

    End Module

End Namespace

